"""Promo-control module."""
from aiogram import types, F, Router, Bot
from aiogram.fsm.context import FSMContext
from aiogram.utils.keyboard import InlineKeyboardBuilder

from manager_cw_bot_api.buttons import Buttons
from manager_cw_bot_api.fsm_handler import (ProcessEnteringPromoST1, ProcessAddNewPromoPromoST2,
                                            ProcessDeletePromoPromoST2)
from manager_cw_bot_api.handler_db_sub_operations import HandlerDB

router_promo: Router = Router()


class HandlerEP:
    """Class for get plus if user has a promo code."""
    __promo_code: str = ""

    def __init__(self, bot: Bot, admin_id: int) -> None:
        self.__bot: Bot = bot
        self.__admin_id: int = admin_id

        router_promo.message.register(
            self.__entering_promo_step1,
            ProcessEnteringPromoST1.promo
        )
        router_promo.message.register(
            self.__add_new_promo_admin_ctrl_step2,
            ProcessAddNewPromoPromoST2.promo
        )
        router_promo.message.register(
            self.__delete_promo_data_admin_ctrl_step2,
            ProcessDeletePromoPromoST2.promo
        )

    async def enter_promo(self, call: types.CallbackQuery, state: FSMContext) -> None:
        """
        Function for get a promo code from user.

        :param call: Callback Query.
        :param state: FSM.

        :return: None.
        """
        await state.set_state(ProcessEnteringPromoST1.promo)
        await self.__bot.edit_message_text(
            chat_id=call.from_user.id,
            text="🎫 Enter a promo code...",
            message_id=call.message.message_id
        )

    async def __entering_promo_step1(self, message: types.Message, state: FSMContext) -> None:
        """
        Get promo code (text format).

        :param message: A promo code.
        :param state: FSM.

        :return: None.
        """
        await state.clear()

        promo: str = message.text
        checked: bool | tuple = await HandlerDB.check_promo_code(promo)
        if checked[0] is False:
            var: InlineKeyboardBuilder = await Buttons.back_on_main()
            await self.__bot.send_message(
                chat_id=message.from_user.id,
                text="❌ *Invalid promo code*.",
                parse_mode="Markdown",
                reply_markup=var.as_markup()
            )
        elif checked[0] is True:
            type_promo: str = checked[3]

            if type_promo == "plus_month":
                count_days: int = 30
            elif type_promo == "plus_five_days":
                count_days: int = 5
            elif type_promo == "plus_one_week":
                count_days: int = 7
            else:
                count_days: int = 0

            var: InlineKeyboardBuilder = await Buttons.sure_apply_promo()
            await self.__bot.send_message(
                chat_id=message.from_user.id,
                text=f"⚠️ Are you sure you want to *apply this promo code* `{promo}`? You'll receive "
                     f"{count_days} days of PLUS.",
                parse_mode="Markdown",
                reply_markup=var.as_markup()
            )

            self.__class__.__promo_code = promo

            router_promo.callback_query.register(
                self.__entering_promo_step2,
                F.data == "apply_promo"
            )

    async def __entering_promo_step2(self, call: types.CallbackQuery) -> None:
        """
        Confirmation of apply a promo code.

        :param call: Callback Query.
        :return: None.
        """
        checked_sub: bool | tuple = await HandlerDB.sub_promo_code(self.__class__.__promo_code)
        if checked_sub[0] is True:
            count_days: int = 0

            checked: tuple = await HandlerDB.check_promo_code(self.__class__.__promo_code)
            if checked[0] is True:

                type_promo: str = checked[3]
                if type_promo == "plus_month":
                    count_days = 30
                elif type_promo == "plus_five_days":
                    count_days = 5
                elif type_promo == "plus_one_week":
                    count_days = 7

            result: bool = await HandlerDB.insert_new_record_for_subscribe(
                message=call,
                days=count_days,
                promo=self.__class__.__promo_code
            )[0]
            if result is True:
                var: InlineKeyboardBuilder = await Buttons.back_on_main()
                await self.__bot.edit_message_text(
                    chat_id=call.from_user.id,
                    text=f"✅ Successful! Done!\nYour PLUS is *activated*!\n\nNow you can:"
                         f"\n- Use 💥 Kandinsky AI | Generate IMG 🖼\n"
                         f"- 🧠 Use AI-Assistance\n- and more; only MORE!",
                    message_id=call.message.message_id,
                    parse_mode="Markdown",
                    reply_markup=var.as_markup()
                )

                await self.__bot.send_message(
                    chat_id=self.__admin_id,
                    text=f"🆕 Admin, PLUS +1 person! 🔥 | With PROMO: `{self.__class__.__promo_code}` | "
                         f"Days: {count_days}.",
                    parse_mode="Markdown",
                    reply_markup=var.as_markup()
                )

                if checked_sub[1] == "none":
                    delete: bool = await HandlerDB.delete_promo_code(self.__class__.__promo_code)[0]
                    if delete:
                        pass

            else:
                await self.__bot.edit_message_text(
                    chat_id=call.from_user.id,
                    text=f"❌ Failed!\nYour PLUS *isn't activated*!\n\nNow you should:"
                         f"\n- write on EMail: help@cwr.su.",
                    message_id=call.message.message_id,
                    parse_mode="Markdown"
                )

        elif checked_sub[0] is False:
            await self.__bot.edit_message_text(
                chat_id=call.from_user.id,
                text=f"❌ Failed!\nYour PLUS *isn't activated*!\n\nNow you should:"
                     f"\n- write on EMail: help@cwr.su.",
                message_id=call.message.message_id,
                parse_mode="Markdown"
            )

    async def show_promo_menu_admin(self, call_query: types.CallbackQuery) -> None:
        """
        Show PROMO Menu. | Control ADMIN.

        :param call_query: Callback Query.

        :return: None.
        """
        var: InlineKeyboardBuilder = await Buttons.get_menu_admin_promos()
        await self.__bot.edit_message_text(
            chat_id=call_query.from_user.id,
            text=f"👑 <b>{call_query.from_user.first_name}</b>, select the item you need below.\n\n",
            message_id=call_query.message.message_id,
            reply_markup=var.as_markup(),
            parse_mode="HTML"
        )

        router_promo.callback_query.register(
            self.__add_new_promo_code_admin_ctrl,
            F.data == "add_new_promo"
        )
        router_promo.callback_query.register(
            self.__show_promo_datas_admin_ctrl,
            F.data == "show_promo_datas"
        )
        router_promo.callback_query.register(
            self.__delete_promo_data_admin_ctrl,
            F.data == "delete_promo_data"
        )

    async def __show_promo_datas_admin_ctrl(self, call: types.CallbackQuery) -> None:
        """
        Show PROMO datas. | Control ADMIN.

        :param call: Callback Query.

        :return: None.
        """
        datas: bool | tuple = await HandlerDB.show_promo_datas()
        var: InlineKeyboardBuilder = await Buttons.back_in_promo_menu()

        if datas[0] is True:
            await self.__bot.edit_message_text(
                chat_id=call.from_user.id,
                text=datas[1],
                message_id=call.message.message_id,
                parse_mode="HTML",
                reply_markup=var.as_markup()
            )
        elif datas[0] is False:
            await self.__bot.edit_message_text(
                chat_id=call.from_user.id,
                text=f"🌐 Promo codes *have not been added*.",
                message_id=call.message.message_id,
                parse_mode="Markdown",
                reply_markup=var.as_markup()
            )

    async def __add_new_promo_code_admin_ctrl(self, call: types.CallbackQuery) -> None:
        """
        Add a new promo. | Control ADMIN. | Confirmation.

        :param call: Callback Query.

        :return: None.
        """
        var: InlineKeyboardBuilder = await Buttons.sure_add_new_promo()
        await self.__bot.edit_message_text(
            chat_id=call.from_user.id,
            text=f"⚠️ Are you sure you want to *add a new promo code*?",
            message_id=call.message.message_id,
            parse_mode="Markdown",
            reply_markup=var.as_markup()
        )

        router_promo.callback_query.register(
            self.__add_new_promo_admin_ctrl_step1,
            F.data == "add_new_promo_confirmation"
        )

    async def __add_new_promo_admin_ctrl_step1(self, call: types.CallbackQuery, state: FSMContext) -> None:
        """
        Add a new promo. | Control ADMIN. | Step 1.

        :param call: Callback Query.
        :param state: FSM.

        :return: None.
        """
        await state.set_state(ProcessAddNewPromoPromoST2.promo)
        await self.__bot.edit_message_text(
            chat_id=call.from_user.id,
            text=f"👌🏻 *{call.from_user.first_name}*, OK. Create a new promo-data explore this format:\n\n"
                 f"*FORMAT*:\n"
                 f"```Example NEW_PROMO_CODE ~ NUM_USES ~ TYPE_PROMO```\n\n"
                 f"Types of promo codes:\n"
                 f"1. `plus_one_week`;\n"
                 f"2. `plus_five_days`;\n"
                 f"3. `plus_month`;\n",
            message_id=call.message.message_id,
            parse_mode="Markdown"
        )

    async def __add_new_promo_admin_ctrl_step2(self, message: types.Message, state: FSMContext) -> None:
        """
        Add a new promo. | Control ADMIN. | Step 2.

        :param message: Message from the admin with PROMO-data.
        :param state: FSM.

        :return: None.
        """
        await state.clear()

        var: InlineKeyboardBuilder = await Buttons.try_again_add_new_promo_or_back_on_main()
        if len(message.text.split(" ~ ")) == 3:
            promo, num_uses, type_promo = message.text.split(" ~ ")
            if int(num_uses) > 0 and len(promo) >= 3:
                checked: tuple = await HandlerDB.add_new_promo_code(promo, num_uses, type_promo)
                if checked[0] is True:
                    var: InlineKeyboardBuilder = await Buttons.back_in_promo_menu()
                    await self.__bot.send_message(
                        chat_id=message.from_user.id,
                        text=f"✅ *Successful*! Done!\n\n{message.from_user.first_name}, *the promo data has been "
                             f"added*!",
                        parse_mode="Markdown",
                        reply_markup=var.as_markup()
                    )
                elif checked[0] is False:
                    if checked[1] == "already_exist":
                        var: InlineKeyboardBuilder = await Buttons.back_in_promo_menu()
                        await self.__bot.send_message(
                            chat_id=message.from_user.id,
                            text=f"❌ *{message.from_user.first_name}*, the promo data *already exists*!",
                            parse_mode="Markdown",
                            reply_markup=var.as_markup()
                        )
                    elif checked[1] == "invalid_type":
                        await self.__bot.send_message(
                            chat_id=message.from_user.id,
                            text=f"❌ *{message.from_user.first_name}*, the type of promo is *invalid*!",
                            parse_mode="Markdown",
                            reply_markup=var.as_markup()
                        )
            else:
                await self.__bot.send_message(
                    chat_id=message.from_user.id,
                    text=f"❌ *{message.from_user.first_name}*, sorry. But i can't add your PROMO data!\n\n"
                         f"*Number of uses* must be *>= 1* and *length* of promo code must be *>= 3*!\n",
                    parse_mode="Markdown",
                    reply_markup=var.as_markup()
                )

        else:
            await self.__bot.send_message(
                chat_id=message.from_user.id,
                text=f"❌ *{message.from_user.first_name}*, sorry. But i can't split your message!\n\n"
                     f"*FORMAT*:\n"
                     f"```Example NEW_PROMO_CODE ~ NUM_USES ~ TYPE_PROMO```\n\n"
                     f"Types of promo codes:\n"
                     f"1. `plus_one_week`;\n"
                     f"2. `plus_five_days`;\n"
                     f"3. `plus_month`;\n",
                parse_mode="Markdown",
                reply_markup=var.as_markup()
            )

            router_promo.callback_query.register(
                self.__add_new_promo_admin_ctrl_step1,
                F.data == "try_again_add_new_promo"
            )

    async def __delete_promo_data_admin_ctrl(self, call: types.CallbackQuery) -> None:
        """
        Delete promo. | Control ADMIN. | Confirmation.

        :param call: Callback Query.

        :return: None.
        """
        var: InlineKeyboardBuilder = await Buttons.sure_delete_promo()
        await self.__bot.edit_message_text(
            chat_id=call.from_user.id,
            text=f"⚠️ Are you sure you want to *delete a promo code*?",
            message_id=call.message.message_id,
            parse_mode="Markdown",
            reply_markup=var.as_markup()
        )

        router_promo.callback_query.register(
            self.__delete_promo_data_admin_ctrl_step1,
            F.data == "delete_promo_confirmation"
        )

    async def __delete_promo_data_admin_ctrl_step1(
            self,
            call: types.CallbackQuery,
            state: FSMContext
    ) -> None:
        """
        Delete promo. | Control ADMIN. | Step 1.

        :param call: Callback Query.
        :param state: FSM.

        :return: None.
        """
        await state.set_state(ProcessDeletePromoPromoST2.promo)
        await self.__bot.edit_message_text(
            chat_id=call.from_user.id,
            text=f"👌🏻 *{call.from_user.first_name}*, OK. Send me the promo code that you want to delete.\n",
            message_id=call.message.message_id,
            parse_mode="Markdown"
        )

    async def __delete_promo_data_admin_ctrl_step2(
            self,
            message: types.Message,
            state: FSMContext
    ) -> None:
        """
        Delete promo. | Control ADMIN. | Step 2.

        :param message: Message from the admin with PROMO-data.
        :param state: FSM.

        :return: None.
        """
        await state.clear()

        promo: str = message.text
        checked: bool = await HandlerDB.delete_promo_code(promo)
        if checked is True:
            var: InlineKeyboardBuilder = await Buttons.back_in_promo_menu()
            await self.__bot.send_message(
                chat_id=message.from_user.id,
                text=f"✅ *Successful*! Done!\n\n{message.from_user.first_name}, *the promo data has been deleted*!",
                parse_mode="Markdown",
                reply_markup=var.as_markup()
            )
        else:
            var: InlineKeyboardBuilder = await Buttons.try_again_delete_promo_or_back_on_main()
            await self.__bot.send_message(
                chat_id=message.from_user.id,
                text=f"❌ *{message.from_user.first_name}*, the promo data *isn't exists*!\n\n"
                     f"Or there was some kind of error when deleting the record. ANSWER: (checked) {checked}.",
                parse_mode="Markdown",
                reply_markup=var.as_markup()
            )

            router_promo.callback_query.register(
                self.__delete_promo_data_admin_ctrl_step1,
                F.data == "try_again_delete_promo"
            )
